{VERSION 1 0 "X11/Motif" "1.0"}{GLOBALS 3 1}{FONT 0 "-adobe-helve
tica-bold-r-normal--*-140-*" "helvetica" "Helvetica-Bold" 8 14 0 
"Helvetica-Bold" 12}{FONT 1 "-adobe-times-medium-r-normal--*-140-
*" "times" "Times-Roman" 4 14 64 "Times-Roman" 12}{FONT 2 "-adobe
-courier-medium-r-normal--*-140-*" "courier" "Courier" 4 14 192 "
Courier" 12}{SCP_R 1 0 68{COM_R 2 0{TEXT 1 19 "Cantilevered Block
s"}}{COM_R 3 0{TEXT 1 242 "\015This worksheet is designed to acco
mpany Chapter 4 of \"Introduction to Scientific Programming: Comp
utational Problem Solving Using Maple and C\" by Joseph L. Zachar
y.  In it, we will use Maple to explore the block-stacking proble
m.  (08Oct96)\015"}}{SEP_R 4 0}{SEP_R 5 0}{COM_R 6 0{TEXT 1 272 "
\015Getting Started.  To use this worksheet you will need to use \+
some Maple extensions that we have created.  Read in our blocks p
ackage by evaluating the Maple command below.  (You will need to \+
have first installed our custom Maple library and configured Mapl
e to use it.) \015"}}{INP_R 7 0 "> "{TEXT 0 13 "with(blocks);"}}
{SEP_R 8 0}{SEP_R 9 0}{COM_R 10 0{TEXT 1 397 "\015Calculating Ext
ensions.  The three functions defined in the library are describe
d in Chapter 4 of the text.  Each takes as its parameter the numb
er of blocks in a stack of blocks arranged as described in the te
xt and reports back the amount by which the top block extends bey
ond the table.  For example, let's compute (in three different wa
ys) the extension that can be achieved with 100 blocks.\015"}}
{COM_R 11 0{TEXT 1 148 "The \"blockFloat\" function uses floating
-point arithmetic to explicitly sum the series 1/2 + 1/4 + 1/6 + \+
1/8 + ... + 1/2n.  For n = 100, that sum is\012"}}{INP_R 12 0 "> \+
"{TEXT 0 16 "blockFloat(100);"}}{COM_R 13 0{TEXT 1 289 "\012The \
"blockRat\" function uses rational arithmetic to sum the same ser
ies.  After it has computed an exact sum, we use \"evalf\" to con
vert the result into a floating-point number.  The difference bet
ween this sum, and the one calculated with \"blockRat\", can be a
ttributed to roundoff error.\012"}}{INP_R 14 0 "> "{TEXT 0 21 "ev
alf(blockRat(100));"}}{COM_R 15 0{TEXT 1 194 "\012The \"blockFast
\" function computes the sum of the series, not by adding up each
 term, but by using the formula (1/2) (Psi(n+1) + gamma).  Notice
 that we get the same answer as with \"blockFast\".\012"}}{INP_R 
16 0 "> "{TEXT 0 15 "blockFast(100);"}}{COM_R 17 0{TEXT 1 215 "\0
12\"blockFloat\" can be used to study how roundoff error can accu
mulate in a series of computations.  Let's begin by setting Digit
s to 4, which means that we will be computing with four-digit flo
ating-point mantissas.\012"}}{INP_R 18 0 "> "{TEXT 0 12 "Digits :
= 4;"}}{COM_R 19 0{TEXT 1 105 "\012We can now compare the results
 of calling the three functions for increasingly large numbers of
 blocks.\012"}}{INP_R 20 0 "> "{TEXT 0 16 "blockFloat(100);"}}
{INP_R 21 0 "> "{TEXT 0 21 "evalf(blockRat(100));"}}{INP_R 22 0 "
> "{TEXT 0 15 "blockFast(100);"}}{COM_R 23 0{TEXT 1 0 ""}}{INP_R 
24 0 "> "{TEXT 0 16 "blockFloat(200);"}}{INP_R 25 0 "> "{TEXT 0 
21 "evalf(blockRat(200));"}}{INP_R 26 0 "> "{TEXT 0 15 "blockFast
(200);"}}{COM_R 27 0{TEXT 1 0 ""}}{INP_R 28 0 "> "{TEXT 0 16 "blo
ckFloat(400);"}}{INP_R 29 0 "> "{TEXT 0 21 "evalf(blockRat(400));
"}}{INP_R 30 0 "> "{TEXT 0 15 "blockFast(400);"}}{COM_R 31 0{TEXT
 1 0 ""}}{INP_R 32 0 "> "{TEXT 0 16 "blockFloat(800);"}}{INP_R 33
 0 "> "{TEXT 0 21 "evalf(blockRat(800));"}}{INP_R 34 0 "> "{TEXT 
0 15 "blockFast(800);"}}{COM_R 35 0{TEXT 1 0 ""}}{INP_R 36 0 "> "
{TEXT 0 17 "blockFloat(1600);"}}{INP_R 37 0 "> "{TEXT 0 22 "evalf
(blockRat(1600));"}}{INP_R 38 0 "> "{TEXT 0 16 "blockFast(1600);"
}}{COM_R 39 0{TEXT 1 778 "\012Notice what happens:\015\015* block
Float, which operates by directly summing the series using floati
ng-point arithmetic, suffers more and more from roundoff error as
 the number of terms increases.\015\015* blockRat, which operates
 by directly summing the series using rational number arithmetic,
 always returns an exact answer.  We then convert that exact answ
er into a floating-point number that is exact to the available nu
mber of digits.\015\015* blockFast, which operates by exploiting \+
specialized mathematical knowledge about the Harmonic series, ret
urns a number that is exact to the available number of digits.\01
5\015* blockRat is much slower than blockFloat for large series, \+
but neither is practical for dealing with extremely large series.
  blockFast, on the other hand, is extremely fast.\015"}}{SEP_R 
40 0}{COM_R 41 0{TEXT 1 149 "\015Exercises.  Let's go back to 10-
digit mantissas and explore how roundoff error accumulates when d
oing a long series of floating-point calculations.\012"}}{INP_R 
42 0 "> "{TEXT 0 13 "Digits := 10;"}}{COM_R 43 0{TEXT 1 126 "\012
Using blockFast, we can find the extension possible with ten thou
sand blocks.  (We save the result in the variable \"truth\".)\012
"}}{INP_R 44 0 "> "{TEXT 0 26 "truth := blockFast(10000);"}}
{COM_R 45 0{TEXT 1 137 "\012Let's use blockFloat to do the same c
alculation using mantissa lengths ranging from 1 to 10.  For exam
ple, with a five-digit mantissa,\015 "}}{INP_R 46 0 "> "{TEXT 0 
12 "Digits := 5;"}}{COM_R 47 0{TEXT 1 75 "\012the result is as fo
llows.  (We save the result in the variable \"result\".)\012"}}
{INP_R 48 0 "> "{TEXT 0 28 "result := blockFloat(10000);"}}{COM_R
 49 0{TEXT 1 37 "\012and after going back to ten digits \015"}}
{INP_R 50 0 "> "{TEXT 0 13 "Digits := 10;"}}{COM_R 51 0{TEXT 1 56
 "\012we find the that relative error is approximately 2.8%.\012"
}}{INP_R 52 0 "> "{TEXT 0 28 "abs(truth - result) / truth;"}}
{COM_R 53 0{TEXT 1 247 "\012You should do similar calculations us
ing for each mantissa length from 1 to 10.  When you are done, yo
u will know the relative error for each mantissa length that is e
xhibited by blockFloat when calculating the extension for ten tho
usand blocks.\015"}}{COM_R 54 0{TEXT 1 476 "You can visualize the
 effect of mantissa length on relative error by plotting mantissa
 length on the x-axis against relative error on the y-axis.  To d
o this, use the command below.  It plots the numbers 1 through 10
 against some arbitrarily chosen numbers.  You will have to repla
ce the arbitrarily chosen numbers with the relative errors that y
ou have calculated.  For example, [5, 30] would become [5, .0277]
.  (Rounding the relative errors to three decimal places is OK.)\
012"}}{INP_R 55 0 "> "{TEXT 0 264 "plot([[1, 88], [2, 22], [3, 14
], [4, 13], [5, 30],\012          [6, 0], [7, 2], [8, 99], [9, 17
], [10, 88]],      \012        0..10, style=POINT, symbol=BOX,\01
2        title = `Relative error when computing blockFloat(100)`,
\012        labels = [`Mantissa length`, `Error`]);"}}{COM_R 56 0
{TEXT 1 312 "\015(The plot will show up in a separate window.  Yo
u can paste it back into your worksheet if you wish.  Just pull d
own the plot's \"Edit\" menu and select \"Copy\".  Then use the m
ouse to put the cursor inside of this text block, pull down the w
orksheet's \"Edit\" menu, and select \"Paste\".  The plot will app
ear below.)"}}{COM_R 57 0{TEXT 1 167 "\015It is also interesting \+
to compare the amount of time required by blockFloat and blockRat
 to do their calculations.  Let's begin by going back to ten-digi
t mantissas.\012"}}{INP_R 58 0 "> "{TEXT 0 13 "Digits := 10;"}}
{COM_R 59 0{TEXT 1 332 "\012For block stack sizes of 100, 200, 40
0, 800, 1600, and 3200, let's calculate the time required to calc
ulate the extension using both blockFloat and blockRat.  For exam
ple, the time required to compute blockFloat(100) is the second n
umber displayed by this sequence of commands.  (The first is the \+
result of the call to blockFloat.)\012"}}{INP_R 60 0 "> "{TEXT 0 
49 "start := time(): blockFloat(100); time() - start;"}}{COM_R 61
 0{TEXT 1 281 "\012The timing function is so crude that for quick
 calculations, it is a good idea to do several calculations and c
alculate an average.  To obtain a better timing result for blockF
loat(100), for example, we can do eight identical calculations an
d then divide the overall result by 8.\012"}}{INP_R 62 0 "> "
{TEXT 0 174 "start := time(): blockFloat(100): blockFloat(100): b
lockFloat(100):\012blockFloat(100): blockFloat(100): blockFloat(1
00): blockFloat(100):\012blockFloat(100); (time() - start) / 8;"}
}{COM_R 63 0{TEXT 1 126 "\012For long calculations such as blockF
loat(3200), of course, this would be unnecessary and would take a
n extremely long time. \015"}}{COM_R 64 0{TEXT 1 221 "You should \+
time blockFloat for stack sizes of 100, 200, 400, 800, 1600, and \+
3200.  You can plot your results by completing the following comm
and.  You will need to insert the times that you obtain in place \+
of the zeroes.\012"}}{INP_R 65 0 "> "{TEXT 0 210 "plot([[100, 0],
 [200, 0], [400, 0], [800, 0], [1600, 0], [3200, 0]],\012        \+
0..3200, style=POINT, symbol=BOX,\012        title = `Time in sec
onds required by blockFloat`,\012        labels = [`Stack size`, \+
`Time`]);"}}{COM_R 66 0{TEXT 1 136 "\012You should now repeat the
 timing experiments using blockRat instead of blockFloat.  Plot y
our results by completing the command below.\012"}}{INP_R 67 0 ">
 "{TEXT 0 208 "plot([[100, 0], [200, 0], [400, 0], [800, 0], [160
0, 0], [3200, 0]],\012        0..3200, style=POINT, symbol=BOX,\0
12        title = `Time in seconds required by blockRat`,\012    \+
    labels = [`Stack size`, `Time`]);"}}{COM_R 68 0{TEXT 1 88 "\0
12Compare the shapes of the two curves that you create and try to
 explain the difference."}}{SEP_R 69 0}}{END}
