{VERSION 1 0 "X11/Motif" "1.0"}{GLOBALS 3 1}{FONT 0 "-adobe-helve
tica-bold-r-normal--*-140-*" "helvetica" "Helvetica-Bold" 8 14 0 
"Helvetica-Bold" 12}{FONT 1 "-adobe-times-medium-r-normal--*-140-
*" "times" "Times-Roman" 4 14 64 "Times-Roman" 12}{FONT 2 "-adobe
-courier-medium-r-normal--*-140-*" "courier" "Courier" 4 14 192 "
Courier" 12}{SCP_R 1 0 87{COM_R 2 0{TEXT 1 20 "Function Evaluatio
n\015"}}{COM_R 3 0{TEXT 1 253 "This worksheet is designed to acco
mpany Chapter 5 of Introduction to Scientific Programming: Comput
ational Problem Solving Using Maple and C by Joseph L. Zachary.  \+
In it, we will use Maple to explore what happens when a function \+
call is evaluated.  (22\015"}}{SEP_R 4 0}{SEP_R 5 0}{COM_R 6 0
{TEXT 1 287 "\015Getting Started.  To use this worksheet you will
 need to use some extensions to Maple that we have created.  Read
 in our function simulation package by evaluating the Maple comma
nd below.  (You will need to have first installed our custom Mapl
e library and configured Maple to use it.)\012"}}{INP_R 7 0 "> "
{TEXT 0 24 "with(functionSimulator);"}}{SEP_R 8 0}{SEP_R 9 0}
{COM_R 10 0{TEXT 1 330 "\015Function Evaluation.  In this laborat
ory we will be exploring how user-defined functions are defined a
nd what happens when they are called. Recall that a function is d
efined by specifying its name, its formal parameters, and its bod
y.  For example, here is a function called \"avg\" that returns t
he average of its two parameters.\012"}}{INP_R 11 0 "> "{TEXT 0 
28 "avg := (x,y) -> (x + y) / 2;"}}{COM_R 12 0{TEXT 1 144 "\015On
ce we have defined a function in this way, we can use it as if it
 were built in.  For example, we can compute the average of 6+6 a
nd 3*6 via\012"}}{INP_R 13 0 "> "{TEXT 0 14 "avg(6+6, 3*6);"}}
{COM_R 14 0{TEXT 1 130 "\012Here, 6+6 and 3*6 are called the actu
al parameters, to distinguish them from the formal parameters tha
t are used to define \"avg\"."}}{SEP_R 15 0}{COM_R 16 0{TEXT 1 
105 "Maple follows a three-step process when evaluating a call to
 a programmer-defined function.  In the call "}}{INP_R 17 0 "> "
{TEXT 0 14 "avg(6+6, 3*6);"}}{COM_R 18 0{TEXT 1 84 "the expressio
ns \"6+6\" and \"3*6\" are evaluated to obtain \"12\" and \"18\",
 respectively."}}{SEP_R 19 0}{COM_R 20 0{TEXT 1 247 "Maple's seco
nd step is to substitute the evaluated actual parameters for the \+
corresponding formal parameters in the body of the function.  In \+
the case of the call to \"avg\" above, 12 corresponds to x and 18
 corresponds to y.  The body of  \"avg\" is\012"}}{INP_R 21 0 "> \+
"{TEXT 0 18 "1/2 * x + 1/2 * y;"}}{COM_R 22 0{TEXT 1 58 "\012When
 Maple substitutes 12 for x and 18 for y, it obtains\012"}}{INP_R
 23 0 "> "{TEXT 0 20 "1/2 * 12 + 1/2 * 18;"}}{COM_R 24 0{TEXT 1 
239 "\012Maple's third step is to evaluate the expression that re
sults from the substitution just as if it had been typed directly
 into Maple.  Since the result of the expression above is 15, thi
s is also the result of the original function call.\012"}}{INP_R 
25 0 "> "{TEXT 0 14 "(12 + 18) / 2;"}}{COM_R 26 0{TEXT 1 235 "Sim
ulating Function Evaluation When you are first learning how to wr
ite functions, it can be difficult to visualize these three steps
.  We have created a special function called \"functionTrace\" to
 help you do this.  Let's try it out.  "}}{SEP_R 27 0}{COM_R 28 0
{TEXT 1 211 "Suppose you would like to see what really goes on wh
en you make the function call \"avg(6+6, 3*6)\".  You can do this
 by enclosing the function call in backquotes and passing it as a
 parameter to \"functionTrace\".\012"}}{INP_R 29 0 "> "{TEXT 0 31
 "functionTrace(`avg(6+6, 3*6)`);"}}{COM_R 30 0{TEXT 1 447 "\012L
ook at all of the information that is printed out.  We are first \+
shown the formal parameters and the body of \"avg\".  We are then
 shown the results of evaluating the actual parameters, the expre
ssion obtained when the actual parameters are substituted for the
 formal parameters in the body of the function, and the result of
 evaluating this expression.  Best of all, \"functionTrace\" will
 work for any programmer-defined function that you create.\015"}}
{COM_R 31 0{TEXT 1 62 "For example, let's define a function that \+
cubes its argument.\012"}}{INP_R 32 0 "> "{TEXT 0 19 "cube := (x)
 -> x^3;"}}{COM_R 33 0{TEXT 1 98 "\012We can trace Maple's proces
s of evaluating a call to \"cube\" as easily as we did a call of \+
\"avg\".\012"}}{INP_R 34 0 "> "{TEXT 0 26 "functionTrace(`cube(15
)`);"}}{COM_R 35 0{TEXT 1 0 ""}}{SEP_R 36 0}{SEP_R 37 0}{COM_R 38
 0{TEXT 1 143 "\015Common Errors.  We can use \"functionTrace\" t
o illustrate a number of mistakes that beginning programmers tend
 to make when calling functions.\015"}}{SEP_R 39 0}{COM_R 40 0
{TEXT 1 366 "\015The wrong number of parameters.  When you call a
 function, it is important that there be the same number of actua
l parameters in the function call as there are formal parameters \+
in the function definition.  Otherwise, Maple will not be able to
 match each actual to a formal prior to doing the substitution.\0
15\012 Here, for example, we give only one parameter to \"avg\":\
012"}}{INP_R 41 0 "> "{TEXT 0 24 "functionTrace(`avg(5)`);"}}
{COM_R 42 0{TEXT 1 72 "\012This is a mistake, of course.  Here we
 give three parameters to \"avg\":\012"}}{INP_R 43 0 "> "{TEXT 0 
28 "functionTrace(`avg(1,2,3)`);"}}{COM_R 44 0{TEXT 1 109 "\012Th
is is also a mistake.  If we type these function calls directly i
nto Maple instead of into the simulator:\012"}}{INP_R 45 0 "> "
{TEXT 0 7 "avg(5);"}}{COM_R 46 0{TEXT 1 0 ""}}{INP_R 47 0 "> "
{TEXT 0 11 "avg(1,2,3);"}}{COM_R 48 0{TEXT 1 222 "\015we see that
 Maple always complains if you provide too few parameters, but ig
nores the extra parameters if you supply too many.  Nevertheless,
 you should always supply exactly the right number of parameters \+
to a function.\015"}}{SEP_R 49 0}{COM_R 50 0{TEXT 1 317 "\015Magi
c invocations.  For some reason, many beginning students appear t
o expect that the actual parameters will get to the function by m
agic.  One commonly attempted tactic is to try to assign values t
o variables with the same name as the formal parameters and to th
en \"call\" the function by just mentioning its name.\012"}}
{INP_R 51 0 "> "{TEXT 0 8 "x := 12;"}}{INP_R 52 0 "> "{TEXT 0 8 "
y := 18;"}}{INP_R 53 0 "> "{TEXT 0 4 "avg;"}}{COM_R 54 0{TEXT 1 
89 "\012Not surprisingly, this doesn't work!  If we run this func
tion \"call\" through the tracer\012"}}{INP_R 55 0 "> "{TEXT 0 21
 "functionTrace(`avg`);"}}{COM_R 56 0{TEXT 1 39 "\015we discover \+
the nature of our mistake."}}{COM_R 57 0{TEXT 1 80 "\012Of course
, it would work if we passed \"x\" and \"y\" as the two actual pa
rameters\012"}}{INP_R 58 0 "> "{TEXT 0 26 "functionTrace(`avg(x,y
)`);"}}{COM_R 59 0{TEXT 1 44 "\015since they now have the appropr
iate values."}}{COM_R 60 0{TEXT 1 123 "\015Of course, there is no
thing special about the names \"x\" and \"y\".  We could just as \+
easily have used \"first\" and \"second\":\012"}}{INP_R 61 0 "> "
{TEXT 0 12 "first := 12;"}}{INP_R 62 0 "> "{TEXT 0 13 "second := \+
18;"}}{INP_R 63 0 "> "{TEXT 0 36 "functionTrace(`avg(first, secon
d)`);"}}{COM_R 64 0{TEXT 1 88 "\015From the point of view of the \+
tracer, the last two invocations of \"avg\" were identical."}}
{COM_R 65 0{TEXT 1 276 "\015Remember: it doesn't matter how you w
rite the actual parameters--all that matters is what they evaluat
e to.  And when you are using a function, it doesn't matter at al
l what the formal parameters are called--that is pertinent only w
ithin the implementation of the function. \015"}}{SEP_R 66 0}
{SEP_R 67 0}{COM_R 68 0{TEXT 1 68 "\015Let's experiment with a di
fferent programmer-defined function now.\012"}}{INP_R 69 0 "> "
{TEXT 0 33 "distance := (t) -> 1/2 * g * t^2;"}}{COM_R 70 0{TEXT 
1 21 "\012If we call distance\012"}}{INP_R 71 0 "> "{TEXT 0 14 "d
istance(15.);"}}{COM_R 72 0{TEXT 1 364 "\012you will notice that \+
the variable \"g\" appears in the result.  This is because it app
ears in the body of the function definition but is NOT a paramete
r.  As a result, it appears in the substituted body.  When Maple \+
evaluates the substituted body, it tries to look up the value of \+
\"g\" like it would for any variable, and failing to find one it \+
leaves \"g\" as it is.\015"}}{COM_R 73 0{TEXT 1 75 "This will be \+
more evident if we trace the evaluation of the function call.\012
"}}{INP_R 74 0 "> "{TEXT 0 31 "functionTrace(`distance(15.)`);"}}
{COM_R 75 0{TEXT 1 28 "\012If we now give \"g\" a value\012"}}
{INP_R 76 0 "> "{TEXT 0 9 "g := 9.8;"}}{COM_R 77 0{TEXT 1 48 "\01
2and then evaluate the call to \"distance\" again\012"}}{INP_R 78
 0 "> "{TEXT 0 31 "functionTrace(`distance(15.)`);"}}{COM_R 79 0
{TEXT 1 149 "\012we get a numerical answer.  Notice that the valu
e of \"g\" is not looked up until the substituted body is actuall
y evaluated.  Thus, if we change \"g\"\012"}}{INP_R 80 0 "> "
{TEXT 0 10 "g := 15.5;"}}{COM_R 81 0{TEXT 1 23 "\012and try one m
ore time\012"}}{INP_R 82 0 "> "{TEXT 0 31 "functionTrace(`distanc
e(15.)`);"}}{COM_R 83 0{TEXT 1 29 "\015we get a different answer.
 \015"}}{SEP_R 84 0}{SEP_R 85 0}{COM_R 86 0{TEXT 1 355 "\015Exerc
ises. You should now experiment with defining some functions of y
our own.  Implement the following functions.  Test them out by ca
lling them directly from Maple, and also use functionTrace to hel
p you visualize what goes on when they are called.  Remember: to \+
use functionTrace, give it as a single parameter a function call \+
enclosed in backquotes.\015"}}{COM_R 87 0{TEXT 1 628 "(1) \"avg4\
" should take four numbers as its parameters and return their ave
rage. \015\015(2) \"sincos\" should take one number as its parame
ter and return the sum of the sine and cosine of that number. \01
5\015(3) \"miles2inches\" should take a distance in miles as its \+
parameter and return the equivalent distance in inches. \015\015(
4) \"triangle\" should take as its two parameters the measurement
 in degrees of two of the angles in a triangle.  It should return
 the measurement in degrees of the third angle. \015\015(5) \"hyp
ot\" should take as its two parameters the length of the sides of
 a right triangle and should return the length of the hypotenuse.
\015"}}{SEP_R 88 0}}{END}
